@@ -17,6 +17,7 @@
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_data_device.h>
 #include <wlr/types/wlr_foreign_toplevel_management_v1.h>
+#include <wlr/types/wlr_gamma_control_v1.h>
 #include <wlr/types/wlr_input_device.h>
 #include <wlr/types/wlr_keyboard.h>
 #include <wlr/types/wlr_keyboard_group.h>
@@ -174,6 +175,7 @@ struct seat {
 	struct wl_listener swipe_end;
 
 	struct wl_listener request_cursor;
+	struct wl_listener request_set_shape;
 	struct wl_listener request_set_selection;
 	struct wl_listener request_set_primary_selection;
 
@@ -201,6 +203,11 @@ struct server {
 	struct wlr_renderer *renderer;
 	struct wlr_allocator *allocator;
 	struct wlr_backend *backend;
+	struct headless {
+		struct wlr_backend *backend;
+		char pending_output_name[4096];
+	} headless;
+	struct wlr_session *session;
 
 	struct wlr_xdg_shell *xdg_shell;
 	struct wlr_layer_shell_v1 *layer_shell;
@@ -228,6 +235,7 @@ struct server {
 
 	struct seat seat;
 	struct wlr_scene *scene;
+	struct wlr_scene_output_layout *scene_layout;
 
 	/* cursor interactive */
 	enum input_mode input_mode;
@@ -237,10 +245,14 @@ struct server {
 	uint32_t resize_edges;
 
 	/*
-	 * Currently focused view. Updated with each "focus change"
-	 * event. This view is drawn with "active" SSD coloring.
+	 * 'active_view' is generally the view with keyboard-focus, updated with
+	 * each "focus change". This view is drawn with "active" SSD coloring.
+	 *
+	 * The exception is when a layer-shell client takes keyboard-focus in
+	 * which case the currently active view stays active. This is important
+	 * for foreign-toplevel protocol.
 	 */
-	struct view *focused_view;
+	struct view *active_view;
 	/*
 	 * Most recently raised view. Used to avoid unnecessarily
 	 * raising the same view over and over.
@@ -289,6 +301,9 @@ struct server {
 	 */
 	int pending_output_layout_change;
 
+	struct wlr_gamma_control_manager_v1 *gamma_control_manager_v1;
+	struct wl_listener gamma_control_set_gamma;
+
 	struct session_lock *session_lock;
 
 	struct wlr_foreign_toplevel_manager_v1 *foreign_toplevel_manager;
@@ -338,8 +353,10 @@ struct output {
 
 	struct wl_listener destroy;
 	struct wl_listener frame;
+	struct wl_listener request_state;
 
 	bool leased;
+	bool gamma_lut_changed;
 };
 
 #undef LAB_NR_LAYERS
@@ -458,6 +475,8 @@ struct wlr_box output_usable_area_in_layout_coords(struct output *output);
 struct wlr_box output_usable_area_scaled(struct output *output);
 void handle_output_power_manager_set_mode(struct wl_listener *listener,
 	void *data);
+void output_add_virtual(struct server *server, const char *output_name);
+void output_remove_virtual(struct server *server, const char *output_name);
 
 void server_init(struct server *server);
 void server_start(struct server *server);
