@@ -4,6 +4,9 @@
 
 #include <wlr/types/wlr_session_lock_v1.h>
 
+struct output;
+struct server;
+
 struct session_lock {
 	struct wlr_session_lock_v1 *lock;
 	struct wlr_surface *focused;
@@ -18,5 +21,6 @@ struct session_lock {
 
 void session_lock_init(struct server *server);
 void session_lock_output_create(struct session_lock *lock, struct output *output);
+void session_lock_update_for_layout_change(void);
 
 #endif /* LABWC_SESSION_LOCK_H */
