@@ -36,6 +36,7 @@ enum ssd_part_type {
 	LAB_SSD_MENU,
 	LAB_SSD_OSD,
 	LAB_SSD_LAYER_SURFACE,
+	LAB_SSD_LAYER_SUBSURFACE,
 	LAB_SSD_UNMANAGED,
 	LAB_SSD_END_MARKER
 };
@@ -59,10 +60,12 @@ struct wlr_scene_node;
  */
 struct ssd *ssd_create(struct view *view, bool active);
 struct border ssd_get_margin(const struct ssd *ssd);
+void ssd_update_margin(struct ssd *ssd);
 void ssd_set_active(struct ssd *ssd, bool active);
 void ssd_update_title(struct ssd *ssd);
 void ssd_update_geometry(struct ssd *ssd);
 void ssd_destroy(struct ssd *ssd);
+void ssd_titlebar_hide(struct ssd *ssd);
 
 void ssd_enable_keybind_inhibit_indicator(struct ssd *ssd, bool enable);
 
