@@ -70,6 +70,13 @@ enum view_wants_focus {
 struct view;
 struct wlr_surface;
 
+/* Common to struct view and struct xwayland_unmanaged */
+struct mappable {
+	bool connected;
+	struct wl_listener map;
+	struct wl_listener unmap;
+};
+
 /* Basic size hints (subset of XSizeHints from X11) */
 struct view_size_hints {
 	int min_width;
@@ -105,6 +112,8 @@ struct view_impl {
 	struct view_size_hints (*get_size_hints)(struct view *self);
 	/* if not implemented, VIEW_WANTS_FOCUS_ALWAYS is assumed */
 	enum view_wants_focus (*wants_focus)(struct view *self);
+	/* returns true if view reserves space at screen edge */
+	bool (*has_strut_partial)(struct view *self);
 };
 
 struct view {
@@ -142,6 +151,7 @@ struct view {
 	bool minimized;
 	enum view_axis maximized;
 	bool fullscreen;
+	bool visible_on_all_workspaces;
 	enum view_edge tiled;
 	bool inhibits_keybinds;
 	xkb_layout_index_t keyboard_layout;
@@ -194,8 +204,8 @@ struct view {
 		struct wl_listener destroy;
 	} toplevel;
 
-	struct wl_listener map;
-	struct wl_listener unmap;
+	struct mappable mappable;
+
 	struct wl_listener destroy;
 	struct wl_listener surface_destroy;
 	struct wl_listener commit;
@@ -345,6 +355,10 @@ view_is_focusable(struct view *view) {
 	return view_is_focusable_from(view, NULL);
 }
 
+void mappable_connect(struct mappable *mappable, struct wlr_surface *surface,
+	wl_notify_func_t notify_map, wl_notify_func_t notify_unmap);
+void mappable_disconnect(struct mappable *mappable);
+
 void view_toggle_keybinds(struct view *view);
 
 void view_set_activated(struct view *view, bool activated);
@@ -389,8 +403,10 @@ void view_toggle_decorations(struct view *view);
 
 bool view_is_always_on_top(struct view *view);
 bool view_is_always_on_bottom(struct view *view);
+bool view_is_omnipresent(struct view *view);
 void view_toggle_always_on_top(struct view *view);
 void view_toggle_always_on_bottom(struct view *view);
+void view_toggle_visible_on_all_workspaces(struct view *view);
 
 bool view_is_tiled(struct view *view);
 bool view_is_floating(struct view *view);
@@ -417,6 +433,14 @@ void view_append_children(struct view *view, struct wl_array *children);
  */
 bool view_is_related(struct view *view, struct wlr_surface *surface);
 
+/**
+ * view_has_strut_partial() - returns true for views that reserve space
+ * at a screen edge (e.g. panels). These views are treated as if they
+ * have the fixedPosition window rule: i.e. they are not restricted to
+ * the usable area and cannot be moved/resized interactively.
+ */
+bool view_has_strut_partial(struct view *view);
+
 const char *view_get_string_prop(struct view *view, const char *prop);
 void view_update_title(struct view *view);
 void view_update_app_id(struct view *view);
@@ -427,6 +451,7 @@ void view_adjust_size(struct view *view, int *w, int *h);
 
 void view_evacuate_region(struct view *view);
 void view_on_output_destroy(struct view *view);
+void view_connect_map(struct view *view, struct wlr_surface *surface);
 void view_destroy(struct view *view);
 
 enum view_axis view_axis_parse(const char *direction);
