@@ -6,6 +6,8 @@
 #include "view.h"
 
 struct wlr_compositor;
+struct wlr_output;
+struct wlr_output_layout;
 
 struct xwayland_unmanaged {
 	struct server *server;
@@ -13,12 +15,14 @@ struct xwayland_unmanaged {
 	struct wlr_scene_node *node;
 	struct wl_list link;
 
+	struct mappable mappable;
+
+	struct wl_listener associate;
+	struct wl_listener dissociate;
 	struct wl_listener request_activate;
 	struct wl_listener request_configure;
 /*	struct wl_listener request_fullscreen; */
 	struct wl_listener set_geometry;
-	struct wl_listener map;
-	struct wl_listener unmap;
 	struct wl_listener destroy;
 	struct wl_listener set_override_redirect;
 };
@@ -28,11 +32,14 @@ struct xwayland_view {
 	struct wlr_xwayland_surface *xwayland_surface;
 
 	/* Events unique to XWayland views */
+	struct wl_listener associate;
+	struct wl_listener dissociate;
 	struct wl_listener request_activate;
 	struct wl_listener request_configure;
 	struct wl_listener set_class;
 	struct wl_listener set_decorations;
 	struct wl_listener set_override_redirect;
+	struct wl_listener set_strut_partial;
 
 	/* Not (yet) implemented */
 /*	struct wl_listener set_role; */
@@ -54,5 +61,11 @@ void xwayland_server_init(struct server *server,
 	struct wlr_compositor *compositor);
 void xwayland_server_finish(struct server *server);
 
+void xwayland_adjust_usable_area(struct view *view,
+	struct wlr_output_layout *layout, struct wlr_output *output,
+	struct wlr_box *usable);
+
+void xwayland_update_workarea(struct server *server);
+
 #endif /* HAVE_XWAYLAND */
 #endif /* LABWC_XWAYLAND_H */
