@@ -78,6 +78,7 @@ enum action_type {
 	ACTION_TYPE_TOGGLE_DECORATIONS,
 	ACTION_TYPE_TOGGLE_ALWAYS_ON_TOP,
 	ACTION_TYPE_TOGGLE_ALWAYS_ON_BOTTOM,
+	ACTION_TYPE_TOGGLE_OMNIPRESENT,
 	ACTION_TYPE_FOCUS,
 	ACTION_TYPE_UNFOCUS,
 	ACTION_TYPE_ICONIFY,
@@ -87,6 +88,7 @@ enum action_type {
 	ACTION_TYPE_RESIZE,
 	ACTION_TYPE_RESIZE_RELATIVE,
 	ACTION_TYPE_MOVETO,
+	ACTION_TYPE_RESIZETO,
 	ACTION_TYPE_MOVETO_CURSOR,
 	ACTION_TYPE_MOVE_RELATIVE,
 	ACTION_TYPE_SEND_TO_DESKTOP,
@@ -96,6 +98,8 @@ enum action_type {
 	ACTION_TYPE_FOCUS_OUTPUT,
 	ACTION_TYPE_IF,
 	ACTION_TYPE_FOR_EACH,
+	ACTION_TYPE_VIRTUAL_OUTPUT_ADD,
+	ACTION_TYPE_VIRTUAL_OUTPUT_REMOVE,
 };
 
 const char *action_names[] = {
@@ -120,6 +124,7 @@ const char *action_names[] = {
 	"ToggleDecorations",
 	"ToggleAlwaysOnTop",
 	"ToggleAlwaysOnBottom",
+	"ToggleOmnipresent",
 	"Focus",
 	"Unfocus",
 	"Iconify",
@@ -129,6 +134,7 @@ const char *action_names[] = {
 	"Resize",
 	"ResizeRelative",
 	"MoveTo",
+	"ResizeTo",
 	"MoveToCursor",
 	"MoveRelative",
 	"SendToDesktop",
@@ -138,6 +144,8 @@ const char *action_names[] = {
 	"FocusOutput",
 	"If",
 	"ForEach",
+	"VirtualOutputAdd",
+	"VirtualOutputRemove",
 	NULL
 };
 
@@ -325,6 +333,12 @@ action_arg_from_xml_node(struct action *action, const char *nodename, const char
 			goto cleanup;
 		}
 		break;
+	case ACTION_TYPE_RESIZETO:
+		if (!strcmp(argument, "width") || !strcmp(argument, "height")) {
+			action_arg_add_int(action, argument, atoi(content));
+			goto cleanup;
+		}
+		break;
 	case ACTION_TYPE_SEND_TO_DESKTOP:
 		if (!strcmp(argument, "follow")) {
 			action_arg_add_bool(action, argument, parse_bool(content, true));
@@ -353,6 +367,13 @@ action_arg_from_xml_node(struct action *action, const char *nodename, const char
 			goto cleanup;
 		}
 		break;
+	case ACTION_TYPE_VIRTUAL_OUTPUT_ADD:
+	case ACTION_TYPE_VIRTUAL_OUTPUT_REMOVE:
+		if (!strcmp(argument, "output_name")) {
+			action_arg_add_str(action, argument, content);
+			goto cleanup;
+		}
+		break;
 	}
 
 	wlr_log(WLR_ERROR, "Invalid argument for action %s: '%s'",
@@ -578,7 +599,7 @@ view_for_action(struct view *activator, struct server *server,
 		return ctx.view;
 	}
 	default:
-		return server->focused_view;
+		return server->active_view;
 	}
 }
 
@@ -744,6 +765,11 @@ actions_run(struct view *activator, struct server *server,
 				view_toggle_always_on_bottom(view);
 			}
 			break;
+		case ACTION_TYPE_TOGGLE_OMNIPRESENT:
+			if (view) {
+				view_toggle_visible_on_all_workspaces(view);
+			}
+			break;
 		case ACTION_TYPE_FOCUS:
 			if (view) {
 				desktop_focus_view(view, /*raise*/ false);
@@ -794,6 +820,25 @@ actions_run(struct view *activator, struct server *server,
 				view_move(view, x, y);
 			}
 			break;
+		case ACTION_TYPE_RESIZETO:
+			if (view) {
+				int width = action_get_int(action, "width", 0);
+				int height = action_get_int(action, "height", 0);
+
+				/*
+				 * To support only setting one of width/height
+				 * in <action name="ResizeTo" width="" height=""/>
+				 * we fall back to current dimension when unset.
+				 */
+				struct wlr_box box = {
+					.x = view->pending.x,
+					.y = view->pending.y,
+					.width = width ? : view->pending.width,
+					.height = height ? : view->pending.height,
+				};
+				view_move_resize(view, box);
+			}
+			break;
 		case ACTION_TYPE_MOVE_RELATIVE:
 			if (view) {
 				int x = action_get_int(action, "x", 0);
@@ -881,6 +926,20 @@ actions_run(struct view *activator, struct server *server,
 				wl_array_release(&views);
 			}
 			break;
+		case ACTION_TYPE_VIRTUAL_OUTPUT_ADD:
+			{
+				const char *output_name = action_get_str(action, "output_name",
+						NULL);
+				output_add_virtual(server, output_name);
+			}
+			break;
+		case ACTION_TYPE_VIRTUAL_OUTPUT_REMOVE:
+			{
+				const char *output_name = action_get_str(action, "output_name",
+						NULL);
+				output_remove_virtual(server, output_name);
+			}
+			break;
 		case ACTION_TYPE_INVALID:
 			wlr_log(WLR_ERROR, "Not executing unknown action");
 			break;
