@@ -13,10 +13,14 @@ parse_bool(const char *str, int default_value)
 		return true;
 	} else if (!strcasecmp(str, "true")) {
 		return true;
+	} else if (!strcasecmp(str, "on")) {
+		return true;
 	} else if (!strcasecmp(str, "no")) {
 		return false;
 	} else if (!strcasecmp(str, "false")) {
 		return false;
+	} else if (!strcasecmp(str, "off")) {
+		return false;
 	}
 error_not_a_boolean:
 	wlr_log(WLR_ERROR, "(%s) is not a boolean value", str);
