@@ -4,6 +4,7 @@
 #include <linux/input-event-codes.h>
 #include <sys/time.h>
 #include <time.h>
+#include <wlr/types/wlr_cursor_shape_v1.h>
 #include <wlr/types/wlr_primary_selection.h>
 #include <wlr/util/region.h>
 #include "action.h"
@@ -22,6 +23,8 @@
 #include "ssd.h"
 #include "view.h"
 
+#define LAB_CURSOR_SHAPE_V1_VERSION 1
+
 static const char * const *cursor_names = NULL;
 
 /* Usual cursor names */
@@ -88,6 +91,8 @@ cursor_get_from_edge(uint32_t resize_edges)
 			"Failed to resolve wlroots edge %u to cursor name", resize_edges);
 		assert(false);
 	}
+
+	return LAB_CURSOR_DEFAULT;
 }
 
 static enum lab_cursors
@@ -100,11 +105,11 @@ cursor_get_from_ssd(enum ssd_part_type view_area)
 static struct wlr_surface *
 get_toplevel(struct wlr_surface *surface)
 {
-	while (surface && wlr_surface_is_xdg_surface(surface)) {
+	while (surface) {
 		struct wlr_xdg_surface *xdg_surface =
-			wlr_xdg_surface_from_wlr_surface(surface);
+			wlr_xdg_surface_try_from_wlr_surface(surface);
 		if (!xdg_surface) {
-			return NULL;
+			break;
 		}
 
 		switch (xdg_surface->role) {
@@ -117,7 +122,7 @@ get_toplevel(struct wlr_surface *surface)
 			continue;
 		}
 	}
-	if (surface && wlr_surface_is_layer_surface(surface)) {
+	if (surface && wlr_layer_surface_v1_try_from_wlr_surface(surface)) {
 		return surface;
 	}
 	return NULL;
@@ -142,7 +147,7 @@ request_cursor_notify(struct wl_listener *listener, void *data)
 		seat->seat->pointer_state.focused_client;
 
 	/*
-	 * This can be sent by any client, so we check to make sure this one is
+	 * This can be sent by any client, so we check to make sure this one
 	 * actually has pointer focus first.
 	 */
 	if (focused_client == event->seat_client) {
@@ -158,6 +163,33 @@ request_cursor_notify(struct wl_listener *listener, void *data)
 	}
 }
 
+static void
+request_set_shape_notify(struct wl_listener *listener, void *data)
+{
+	struct wlr_cursor_shape_manager_v1_request_set_shape_event *event = data;
+	const char *shape_name = wlr_cursor_shape_v1_name(event->shape);
+	struct seat *seat = wl_container_of(listener, seat, request_set_shape);
+	struct wlr_seat_client *focused_client = seat->seat->pointer_state.focused_client;
+
+	/* Prevent setting a cursor image when moving or resizing */
+	if (seat->server->input_mode != LAB_INPUT_STATE_PASSTHROUGH) {
+		return;
+	}
+
+	/*
+	 * This can be sent by any client, so we check to make sure this one
+	 * actually has pointer focus first.
+	 */
+	if (event->seat_client != focused_client) {
+		wlr_log(WLR_INFO, "seat client %p != focused client %p",
+			event->seat_client, focused_client);
+		return;
+	}
+
+	wlr_log(WLR_DEBUG, "set xcursor to shape %s", shape_name);
+	wlr_cursor_set_xcursor(seat->cursor, seat->xcursor_manager, shape_name);
+}
+
 static void
 request_set_selection_notify(struct wl_listener *listener, void *data)
 {
@@ -252,8 +284,8 @@ cursor_set(struct seat *seat, enum lab_cursors cursor)
 		return;
 	}
 
-	wlr_xcursor_manager_set_cursor_image(
-		seat->xcursor_manager, cursor_names[cursor], seat->cursor);
+	wlr_cursor_set_xcursor(seat->cursor, seat->xcursor_manager,
+		cursor_names[cursor]);
 	seat->server_cursor = cursor;
 }
 
@@ -274,8 +306,15 @@ cursor_update_image(struct seat *seat)
 		}
 		return;
 	}
-	wlr_xcursor_manager_set_cursor_image(
-		seat->xcursor_manager, cursor_names[cursor], seat->cursor);
+	/*
+	 * Call wlr_cursor_unset_image() first to force wlroots to
+	 * update the cursor (e.g. for a new output). Otherwise,
+	 * wlr_cursor_set_xcursor() may detect that we are setting the
+	 * same cursor as before, and do nothing.
+	 */
+	wlr_cursor_unset_image(seat->cursor);
+	wlr_cursor_set_xcursor(seat->cursor, seat->xcursor_manager,
+		cursor_names[cursor]);
 }
 
 bool
@@ -329,7 +368,7 @@ process_cursor_motion_out_of_surface(struct server *server, uint32_t time)
 	if (view) {
 		lx = view->current.x;
 		ly = view->current.y;
-	} else if (node && wlr_surface_is_layer_surface(surface)) {
+	} else if (node && wlr_layer_surface_v1_try_from_wlr_surface(surface)) {
 		wlr_scene_node_coords(node, &lx, &ly);
 #if HAVE_XWAYLAND
 	} else if (node && node->parent == server->unmanaged_tree) {
@@ -546,7 +585,7 @@ static void
 warp_cursor_to_constraint_hint(struct seat *seat,
 		struct wlr_pointer_constraint_v1 *constraint)
 {
-	if (!seat->server->focused_view) {
+	if (!seat->server->active_view) {
 		return;
 	}
 
@@ -555,8 +594,8 @@ warp_cursor_to_constraint_hint(struct seat *seat,
 		double sx = constraint->current.cursor_hint.x;
 		double sy = constraint->current.cursor_hint.y;
 		wlr_cursor_warp(seat->cursor, NULL,
-			seat->server->focused_view->current.x + sx,
-			seat->server->focused_view->current.y + sy);
+			seat->server->active_view->current.x + sx,
+			seat->server->active_view->current.y + sy);
 
 		/* Make sure we are not sending unnecessary surface movements */
 		wlr_seat_pointer_warp(seat->seat, sx, sy);
@@ -606,7 +645,7 @@ create_constraint(struct wl_listener *listener, void *data)
 	constraint->destroy.notify = destroy_constraint;
 	wl_signal_add(&wlr_constraint->events.destroy, &constraint->destroy);
 
-	struct view *view = server->focused_view;
+	struct view *view = server->active_view;
 	if (view && view->surface == wlr_constraint->surface) {
 		constrain_cursor(server, wlr_constraint);
 	}
@@ -654,8 +693,8 @@ apply_constraint(struct seat *seat, struct wlr_pointer *pointer, double *x, doub
 	double sx = seat->cursor->x;
 	double sy = seat->cursor->y;
 
-	sx -= seat->server->focused_view->current.x;
-	sy -= seat->server->focused_view->current.y;
+	sx -= seat->server->active_view->current.x;
+	sy -= seat->server->active_view->current.y;
 
 	double sx_confined, sy_confined;
 	if (!wlr_region_confine(&seat->current_constraint->region, sx, sy,
@@ -917,7 +956,7 @@ cursor_button_press(struct seat *seat, struct wlr_pointer_button_event *event)
 	 */
 	if (ctx.type == LAB_SSD_LAYER_SURFACE) {
 		struct wlr_layer_surface_v1 *layer =
-			wlr_layer_surface_v1_from_wlr_surface(ctx.surface);
+			wlr_layer_surface_v1_try_from_wlr_surface(ctx.surface);
 		if (layer && layer->current.keyboard_interactive) {
 			seat_set_focus_layer(seat, layer);
 		}
@@ -1218,6 +1257,18 @@ cursor_init(struct seat *seat)
 	seat->request_cursor.notify = request_cursor_notify;
 	wl_signal_add(&seat->seat->events.request_set_cursor,
 		&seat->request_cursor);
+
+	struct wlr_cursor_shape_manager_v1 *cursor_shape_manager =
+		wlr_cursor_shape_manager_v1_create(seat->server->wl_display,
+			LAB_CURSOR_SHAPE_V1_VERSION);
+	if (!cursor_shape_manager) {
+		wlr_log(WLR_ERROR, "unable to create cursor_shape interface");
+		exit(EXIT_FAILURE);
+	}
+	seat->request_set_shape.notify = request_set_shape_notify;
+	wl_signal_add(&cursor_shape_manager->events.request_set_shape,
+		&seat->request_set_shape);
+
 	seat->request_set_selection.notify = request_set_selection_notify;
 	wl_signal_add(&seat->seat->events.request_set_selection,
 		&seat->request_set_selection);
@@ -1242,6 +1293,7 @@ void cursor_finish(struct seat *seat)
 	touch_finish(seat);
 
 	wl_list_remove(&seat->request_cursor.link);
+	wl_list_remove(&seat->request_set_shape.link);
 	wl_list_remove(&seat->request_set_selection.link);
 
 	wlr_xcursor_manager_destroy(seat->xcursor_manager);
