@@ -9,6 +9,8 @@
 #define _POSIX_C_SOURCE 200809L
 #include <assert.h>
 #include <strings.h>
+#include <wlr/backend/drm.h>
+#include <wlr/backend/headless.h>
 #include <wlr/types/wlr_buffer.h>
 #include <wlr/types/wlr_drm_lease_v1.h>
 #include <wlr/types/wlr_output.h>
@@ -18,21 +20,56 @@
 #include <wlr/util/log.h>
 #include "common/macros.h"
 #include "common/mem.h"
+#include "common/scene-helpers.h"
 #include "labwc.h"
 #include "layers.h"
 #include "node.h"
 #include "regions.h"
 #include "view.h"
+#include "xwayland.h"
 
 static void
 output_frame_notify(struct wl_listener *listener, void *data)
 {
+	/*
+	 * This function is called every time an output is ready to display a
+	 * frame - which is typically at 60 Hz.
+	 */
 	struct output *output = wl_container_of(listener, output, frame);
 	if (!output_is_usable(output)) {
 		return;
 	}
 
-	wlr_scene_output_commit(output->scene_output);
+	struct wlr_output *wlr_output = output->wlr_output;
+	struct server *server = output->server;
+
+	if (output->gamma_lut_changed) {
+		struct wlr_output_state pending;
+		wlr_output_state_init(&pending);
+		if (!wlr_scene_output_build_state(output->scene_output, &pending, NULL)) {
+			return;
+		}
+		output->gamma_lut_changed = false;
+		struct wlr_gamma_control_v1 *gamma_control =
+			wlr_gamma_control_manager_v1_get_control(
+				server->gamma_control_manager_v1, wlr_output);
+		if (!wlr_gamma_control_v1_apply(gamma_control, &pending)) {
+			wlr_output_state_finish(&pending);
+			return;
+		}
+
+		if (!wlr_output_commit_state(output->wlr_output, &pending)) {
+			wlr_gamma_control_v1_send_failed_and_destroy(gamma_control);
+			wlr_output_state_finish(&pending);
+			return;
+		}
+
+		wlr_damage_ring_rotate(&output->scene_output->damage_ring);
+		wlr_output_state_finish(&pending);
+		return;
+	}
+
+	wlr_scene_output_commit(output->scene_output, NULL);
 
 	struct timespec now = { 0 };
 	clock_gettime(CLOCK_MONOTONIC, &now);
@@ -48,6 +85,7 @@ output_destroy_notify(struct wl_listener *listener, void *data)
 	wl_list_remove(&output->link);
 	wl_list_remove(&output->frame.link);
 	wl_list_remove(&output->destroy.link);
+	wl_list_remove(&output->request_state.link);
 
 	for (size_t i = 0; i < ARRAY_SIZE(output->layer_tree); i++) {
 		wlr_scene_node_destroy(&output->layer_tree[i]->node);
@@ -67,9 +105,33 @@ output_destroy_notify(struct wl_listener *listener, void *data)
 			view_on_output_destroy(view);
 		}
 	}
+
+	/*
+	 * Ensure that we don't accidentally try to dereference
+	 * the output pointer in some output event handler like
+	 * set_gamma.
+	 */
+	output->wlr_output->data = NULL;
+
+	/*
+	 * output->scene_output (if still around at this point) is
+	 * destroyed automatically when the wlr_output is destroyed
+	 */
 	free(output);
 }
 
+static void
+output_request_state_notify(struct wl_listener *listener, void *data)
+{
+	/* This ensures nested backends can be resized */
+	struct output *output = wl_container_of(listener, output, request_state);
+	const struct wlr_output_event_request_state *event = data;
+
+	if (!wlr_output_commit_state(output->wlr_output, event->state)) {
+		wlr_log(WLR_ERROR, "Backend requested a new state that could not be applied");
+	}
+}
+
 static void do_output_layout_change(struct server *server);
 
 static bool
@@ -78,6 +140,34 @@ can_reuse_mode(struct wlr_output *wlr_output)
 	return wlr_output->current_mode && wlr_output_test(wlr_output);
 }
 
+static void
+add_output_to_layout(struct server *server, struct output *output)
+{
+	struct wlr_output *wlr_output = output->wlr_output;
+	struct wlr_output_layout_output *layout_output =
+		wlr_output_layout_add_auto(server->output_layout, wlr_output);
+	if (!layout_output) {
+		wlr_log(WLR_ERROR, "unable to add output to layout");
+		return;
+	}
+
+	if (!output->scene_output) {
+		output->scene_output =
+			wlr_scene_output_create(server->scene, wlr_output);
+		if (!output->scene_output) {
+			wlr_log(WLR_ERROR, "unable to create scene output");
+			return;
+		}
+		/*
+		 * Note: wlr_scene_output_layout_add_output() is not
+		 * safe to call twice, so we call it only when initially
+		 * creating the scene_output.
+		 */
+		wlr_scene_output_layout_add_output(server->scene_layout,
+			layout_output, output->scene_output);
+	}
+}
+
 static void
 new_output_notify(struct wl_listener *listener, void *data)
 {
@@ -88,6 +178,12 @@ new_output_notify(struct wl_listener *listener, void *data)
 	struct server *server = wl_container_of(listener, server, new_output);
 	struct wlr_output *wlr_output = data;
 
+	/* Name virtual output */
+	if (wlr_output_is_headless(wlr_output) && server->headless.pending_output_name[0] != '\0') {
+		wlr_output_set_name(wlr_output, server->headless.pending_output_name);
+		server->headless.pending_output_name[0] = '\0';
+	}
+
 	/*
 	 * We offer any display as available for lease, some apps like
 	 * gamescope, want to take ownership of a display when they can
@@ -95,7 +191,7 @@ new_output_notify(struct wl_listener *listener, void *data)
 	 * This is also useful for debugging the DRM parts of
 	 * another compositor.
 	 */
-	if (server->drm_lease_manager) {
+	if (server->drm_lease_manager && wlr_output_is_drm(wlr_output)) {
 		wlr_drm_lease_v1_manager_offer_output(
 			server->drm_lease_manager, wlr_output);
 	}
@@ -180,6 +276,9 @@ new_output_notify(struct wl_listener *listener, void *data)
 	output->frame.notify = output_frame_notify;
 	wl_signal_add(&wlr_output->events.frame, &output->frame);
 
+	output->request_state.notify = output_request_state_notify;
+	wl_signal_add(&wlr_output->events.request_state, &output->request_state);
+
 	wl_list_init(&output->regions);
 
 	/*
@@ -227,9 +326,7 @@ new_output_notify(struct wl_listener *listener, void *data)
 	 */
 	server->pending_output_layout_change++;
 
-	wlr_output_layout_add_auto(server->output_layout, wlr_output);
-	output->scene_output = wlr_scene_get_scene_output(server->scene, wlr_output);
-	assert(output->scene_output);
+	add_output_to_layout(server, output);
 
 	/* Create regions from config */
 	regions_reconfigure_output(output);
@@ -245,6 +342,9 @@ new_output_notify(struct wl_listener *listener, void *data)
 void
 output_init(struct server *server)
 {
+	server->gamma_control_manager_v1 =
+		wlr_gamma_control_manager_v1_create(server->wl_display);
+
 	server->new_output.notify = new_output_notify;
 	wl_signal_add(&server->backend->events.new_output, &server->new_output);
 
@@ -257,7 +357,12 @@ output_init(struct server *server)
 		wlr_log(WLR_ERROR, "unable to create output layout");
 		exit(EXIT_FAILURE);
 	}
-	wlr_scene_attach_output_layout(server->scene, server->output_layout);
+	server->scene_layout = wlr_scene_attach_output_layout(server->scene,
+		server->output_layout);
+	if (!server->scene_layout) {
+		wlr_log(WLR_ERROR, "unable to create scene layout");
+		exit(EXIT_FAILURE);
+	}
 
 	/* Enable screen recording with wf-recorder */
 	wlr_xdg_output_manager_v1_create(server->wl_display,
@@ -320,24 +425,34 @@ output_config_apply(struct server *server,
 
 		/* Only do Layout specific actions if the commit went trough */
 		if (need_to_add) {
-			wlr_output_layout_add_auto(server->output_layout, o);
-			output->scene_output =
-				wlr_scene_get_scene_output(server->scene, o);
-			assert(output->scene_output);
+			add_output_to_layout(server, output);
 		}
 
 		if (output_enabled) {
 			struct wlr_box pos = {0};
 			wlr_output_layout_get_box(server->output_layout, o, &pos);
 			if (pos.x != head->state.x || pos.y != head->state.y) {
-				/* This overrides the automatic layout */
-				wlr_output_layout_move(server->output_layout, o,
+				/*
+				 * This overrides the automatic layout
+				 *
+				 * wlr_output_layout_add() in fact means _move()
+				 */
+				wlr_output_layout_add(server->output_layout, o,
 					head->state.x, head->state.y);
 			}
 		}
 
 		if (need_to_remove) {
 			regions_evacuate_output(output);
+			/*
+			 * At time of writing, wlr_output_layout_remove()
+			 * indirectly destroys the wlr_scene_output, but
+			 * this behavior may change in future. To remove
+			 * doubt and avoid either a leak or double-free,
+			 * explicitly destroy the wlr_scene_output before
+			 * calling wlr_output_layout_remove().
+			 */
+			wlr_scene_output_destroy(output->scene_output);
 			wlr_output_layout_remove(server->output_layout, o);
 			output->scene_output = NULL;
 		}
@@ -445,6 +560,20 @@ handle_output_layout_change(struct wl_listener *listener, void *data)
 	do_output_layout_change(server);
 }
 
+static void
+handle_gamma_control_set_gamma(struct wl_listener *listener, void *data)
+{
+	struct server *server = wl_container_of(listener, server, gamma_control_set_gamma);
+	const struct wlr_gamma_control_manager_v1_set_gamma_event *event = data;
+
+	struct output *output = event->output->data;
+	if (!output_is_usable(output)) {
+		return;
+	}
+	output->gamma_lut_changed = true;
+	wlr_output_schedule_frame(output->wlr_output);
+}
+
 void
 output_manager_init(struct server *server)
 {
@@ -457,6 +586,10 @@ output_manager_init(struct server *server)
 	server->output_manager_apply.notify = handle_output_manager_apply;
 	wl_signal_add(&server->output_manager->events.apply,
 		&server->output_manager_apply);
+
+	server->gamma_control_set_gamma.notify = handle_gamma_control_set_gamma;
+	wl_signal_add(&server->gamma_control_manager_v1->events.set_gamma,
+		&server->gamma_control_set_gamma);
 }
 
 struct output *
@@ -519,6 +652,16 @@ update_usable_area(struct output *output)
 	struct wlr_box old = output->usable_area;
 	layers_arrange(output);
 
+#if HAVE_XWAYLAND
+	struct view *view;
+	wl_list_for_each(view, &output->server->views, link) {
+		if (view->mapped && view->type == LAB_XWAYLAND_VIEW) {
+			xwayland_adjust_usable_area(view,
+				output->server->output_layout,
+				output->wlr_output, &output->usable_area);
+		}
+	}
+#endif
 	return !wlr_box_equal(&old, &output->usable_area);
 }
 
@@ -527,6 +670,9 @@ output_update_usable_area(struct output *output)
 {
 	if (update_usable_area(output)) {
 		regions_update_geometry(output);
+#if HAVE_XWAYLAND
+		xwayland_update_workarea(output->server);
+#endif
 		desktop_arrange_all_views(output->server);
 	}
 }
@@ -546,6 +692,9 @@ output_update_all_usable_areas(struct server *server, bool layout_changed)
 		}
 	}
 	if (usable_area_changed || layout_changed) {
+#if HAVE_XWAYLAND
+		xwayland_update_workarea(server);
+#endif
 		desktop_arrange_all_views(server);
 	}
 }
@@ -609,3 +758,56 @@ handle_output_power_manager_set_mode(struct wl_listener *listener, void *data)
 		break;
 	}
 }
+
+void
+output_add_virtual(struct server *server, const char *output_name)
+{
+	if (output_name) {
+		/* Prevent creating outputs with the same name */
+		struct output *output;
+		wl_list_for_each(output, &server->outputs, link) {
+			if (wlr_output_is_headless(output->wlr_output) &&
+					!strcmp(output->wlr_output->name, output_name)) {
+				wlr_log(WLR_DEBUG,
+					"refusing to create virtual output with duplicate name");
+				return;
+			}
+		}
+		snprintf(server->headless.pending_output_name,
+			sizeof(server->headless.pending_output_name), "%s", output_name);
+	} else {
+		server->headless.pending_output_name[0] = '\0';
+	}
+	/*
+	 * Setting it to (0, 0) here disallows changing resolution from tools like
+	 * wlr-randr (returns error)
+	 */
+	wlr_headless_add_output(server->headless.backend, 1920, 1080);
+}
+
+void
+output_remove_virtual(struct server *server, const char *output_name)
+{
+	struct output *output;
+	wl_list_for_each(output, &server->outputs, link) {
+		if (wlr_output_is_headless(output->wlr_output)) {
+			if (output_name) {
+				/*
+				 * Given virtual output name, find and destroy virtual output by
+				 * that name.
+				 */
+				if (!strcmp(output->wlr_output->name, output_name)) {
+					wlr_output_destroy(output->wlr_output);
+					return;
+				}
+			} else {
+				/*
+				 * When virtual output name was no supplied by user, simply
+				 * destroy the first virtual output found.
+				 */
+				wlr_output_destroy(output->wlr_output);
+				return;
+			}
+		}
+	}
+}
