@@ -3,8 +3,11 @@
 #include "config.h"
 #include <signal.h>
 #include <sys/wait.h>
+#include <wlr/backend/headless.h>
+#include <wlr/backend/multi.h>
 #include <wlr/types/wlr_data_control_v1.h>
 #include <wlr/types/wlr_export_dmabuf_v1.h>
+#include <wlr/types/wlr_fractional_scale_v1.h>
 #include <wlr/types/wlr_gamma_control_v1.h>
 #include <wlr/types/wlr_input_inhibitor.h>
 #include <wlr/types/wlr_presentation_time.h>
@@ -30,6 +33,9 @@
 #include "workspaces.h"
 #include "xwayland.h"
 
+#define LAB_WLR_COMPOSITOR_VERSION 5
+#define LAB_WLR_FRACTIONAL_SCALE_V1_VERSION 1
+
 static struct wlr_compositor *compositor;
 static struct wl_event_source *sighup_source;
 static struct wl_event_source *sigint_source;
@@ -244,13 +250,30 @@ server_init(struct server *server)
 	 * backend based on the current environment, such as opening an x11
 	 * window if an x11 server is running.
 	 */
-	server->backend = wlr_backend_autocreate(server->wl_display);
+	server->backend = wlr_backend_autocreate(
+		server->wl_display, &server->session);
 	if (!server->backend) {
 		wlr_log(WLR_ERROR, "unable to create backend");
 		fprintf(stderr, helpful_seat_error_message);
 		exit(EXIT_FAILURE);
 	}
 
+	/* Create headless backend to enable adding virtual outputs later on */
+	server->headless.backend = wlr_headless_backend_create(server->wl_display);
+	if (!server->headless.backend) {
+		wlr_log(WLR_ERROR, "unable to create headless backend");
+		exit(EXIT_FAILURE);
+	}
+	wlr_multi_backend_add(server->backend, server->headless.backend);
+
+	/*
+	 * If we don't populate headless backend with a virtual output (that we
+	 * create and immediately destroy), then virtual outputs being added
+	 * later do not work properly when overlaid on real output. Content is
+	 * drawn on the virtual output, but not drawn on the real output.
+	 */
+	wlr_output_destroy(wlr_headless_add_output(server->headless.backend, 0, 0));
+
 	/*
 	 * Autocreates a renderer, either Pixman, GLES2 or Vulkan for us. The
 	 * user can also specify a renderer using the WLR_RENDERER env var.
@@ -329,8 +352,8 @@ server_init(struct server *server)
 	 * room for you to dig your fingers in and play with their behavior if
 	 * you want.
 	 */
-	compositor =
-		wlr_compositor_create(server->wl_display, server->renderer);
+	compositor = wlr_compositor_create(server->wl_display,
+		LAB_WLR_COMPOSITOR_VERSION, server->renderer);
 	if (!compositor) {
 		wlr_log(WLR_ERROR, "unable to create the wlroots compositor");
 		exit(EXIT_FAILURE);
@@ -372,9 +395,10 @@ server_init(struct server *server)
 	wlr_export_dmabuf_manager_v1_create(server->wl_display);
 	wlr_screencopy_manager_v1_create(server->wl_display);
 	wlr_data_control_manager_v1_create(server->wl_display);
-	wlr_gamma_control_manager_v1_create(server->wl_display);
 	wlr_viewporter_create(server->wl_display);
 	wlr_single_pixel_buffer_manager_v1_create(server->wl_display);
+	wlr_fractional_scale_manager_v1_create(server->wl_display,
+		LAB_WLR_FRACTIONAL_SCALE_V1_VERSION);
 
 	idle_manager_create(server->wl_display, server->seat.seat);
 
