@@ -82,7 +82,7 @@ struct ssd_part *
 add_scene_button_corner(struct wl_list *part_list, enum ssd_part_type type,
 		enum ssd_part_type corner_type, struct wlr_scene_tree *parent,
 		struct wlr_buffer *corner_buffer, struct wlr_buffer *icon_buffer,
-		int x, struct view *view)
+		struct wlr_buffer *hover_buffer, int x, struct view *view)
 {
 	int offset_x;
 	float invisible[4] = { 0, 0, 0, 0 };
@@ -108,7 +108,7 @@ add_scene_button_corner(struct wl_list *part_list, enum ssd_part_type type,
 		-offset_x, -rc.theme->border_width);
 
 	/* Finally just put a usual theme button on top, using an invisible hitbox */
-	add_scene_button(part_list, type, parent, invisible, icon_buffer, 0, view);
+	add_scene_button(part_list, type, parent, invisible, icon_buffer, hover_buffer, 0, view);
 	return button_root;
 }
 
@@ -141,11 +141,9 @@ get_scale_box(struct wlr_buffer *buffer, double container_width,
 struct ssd_part *
 add_scene_button(struct wl_list *part_list, enum ssd_part_type type,
 		struct wlr_scene_tree *parent, float *bg_color,
-		struct wlr_buffer *icon_buffer, int x, struct view *view)
+		struct wlr_buffer *icon_buffer, struct wlr_buffer *hover_buffer,
+		int x, struct view *view)
 {
-	struct wlr_scene_node *hover;
-	float hover_bg[4] = {0.15f, 0.15f, 0.15f, 0.3f};
-
 	struct ssd_part *button_root = add_scene_part(part_list, type);
 	parent = wlr_scene_tree_create(parent);
 	button_root->node = &parent->node;
@@ -156,29 +154,78 @@ add_scene_button(struct wl_list *part_list, enum ssd_part_type type,
 		SSD_BUTTON_WIDTH, rc.theme->title_height, 0, 0, bg_color);
 
 	/* Icon */
+	struct wlr_scene_tree *icon_tree = wlr_scene_tree_create(parent);
 	struct wlr_box icon_geo = get_scale_box(icon_buffer,
 		SSD_BUTTON_WIDTH, rc.theme->title_height);
 	struct ssd_part *icon_part = add_scene_buffer(part_list, type,
-		parent, icon_buffer, icon_geo.x, icon_geo.y);
+		icon_tree, icon_buffer, icon_geo.x, icon_geo.y);
 
 	/* Make sure big icons are scaled down if necessary */
 	wlr_scene_buffer_set_dest_size(
 		wlr_scene_buffer_from_node(icon_part->node),
 		icon_geo.width, icon_geo.height);
 
-	/* Hover overlay */
-	hover = add_scene_rect(part_list, type, parent, SSD_BUTTON_WIDTH,
-		rc.theme->title_height, 0, 0, hover_bg)->node;
-	wlr_scene_node_set_enabled(hover, false);
+	/* Hover icon */
+	struct wlr_scene_tree *hover_tree = wlr_scene_tree_create(parent);
+	wlr_scene_node_set_enabled(&hover_tree->node, false);
+	struct wlr_box hover_geo = get_scale_box(hover_buffer,
+		SSD_BUTTON_WIDTH, rc.theme->title_height);
+	struct ssd_part *hover_part = add_scene_buffer(part_list, type,
+		hover_tree, hover_buffer, hover_geo.x, hover_geo.y);
+
+	/* Make sure big icons are scaled down if necessary */
+	wlr_scene_buffer_set_dest_size(
+		wlr_scene_buffer_from_node(hover_part->node),
+		hover_geo.width, hover_geo.height);
 
 	struct ssd_button *button = ssd_button_descriptor_create(button_root->node);
 	button->type = type;
 	button->view = view;
-	button->hover = hover;
+	button->normal = icon_part->node;
+	button->hover = hover_part->node;
 	button->background = bg_rect->node;
+	button->toggled = NULL;
+	button->toggled_hover = NULL;
+	button->icon_tree = icon_tree;
+	button->hover_tree = hover_tree;
 	return button_root;
 }
 
+void
+add_toggled_icon(struct wl_list *part_list, enum ssd_part_type type,
+		struct wlr_buffer *icon_buffer, struct wlr_buffer *hover_buffer)
+{
+	struct ssd_part *part = ssd_get_part(part_list, type);
+	struct ssd_button *button = node_ssd_button_from_node(part->node);
+
+	/* Alternate icon */
+	struct wlr_box icon_geo = get_scale_box(icon_buffer,
+		SSD_BUTTON_WIDTH, rc.theme->title_height);
+
+	struct ssd_part *alticon_part = add_scene_buffer(part_list, type,
+		button->icon_tree, icon_buffer, icon_geo.x, icon_geo.y);
+
+	wlr_scene_buffer_set_dest_size(
+		wlr_scene_buffer_from_node(alticon_part->node),
+		icon_geo.width, icon_geo.height);
+
+	wlr_scene_node_set_enabled(alticon_part->node, false);
+
+	struct wlr_box hover_geo = get_scale_box(hover_buffer,
+		SSD_BUTTON_WIDTH, rc.theme->title_height);
+	struct ssd_part *althover_part = add_scene_buffer(part_list, type,
+		button->hover_tree, hover_buffer, hover_geo.x, hover_geo.y);
+
+	wlr_scene_buffer_set_dest_size(
+		wlr_scene_buffer_from_node(althover_part->node),
+		hover_geo.width, hover_geo.height);
+
+	wlr_scene_node_set_enabled(althover_part->node, false);
+
+	button->toggled = alticon_part->node;
+	button->toggled_hover = althover_part->node;
+}
+
 struct ssd_part *
 ssd_get_part(struct wl_list *part_list, enum ssd_part_type type)
 {
