@@ -7,6 +7,7 @@
 
 #define _POSIX_C_SOURCE 200809L
 #include "config.h"
+#include <assert.h>
 #include <cairo.h>
 #include <ctype.h>
 #include <drm_fourcc.h>
@@ -39,7 +40,8 @@
 
 struct button {
 	const char *name;
-	char fallback_button[6];	/* built-in 6x6 button */
+	const char *alt_name;
+	const char *fallback_button;  /* built-in 6x6 button */
 	struct {
 		struct lab_data_buffer **buffer;
 		float *rgba;
@@ -55,59 +57,166 @@ drop(struct lab_data_buffer **buffer)
 	}
 }
 
+static void
+create_hover_fallback(struct theme *theme, struct lab_data_buffer **hover_buffer,
+		struct lab_data_buffer *icon_buffer)
+{
+	assert(icon_buffer);
+	assert(!*hover_buffer);
+
+	struct surface_context icon =
+		get_cairo_surface_from_lab_data_buffer(icon_buffer);
+	int icon_width = cairo_image_surface_get_width(icon.surface);
+	int icon_height = cairo_image_surface_get_height(icon.surface);
+
+	/* TODO: need to somehow respect rounded corners */
+	int width = SSD_BUTTON_WIDTH;
+	int height = theme->title_height;
+
+	if (width && height) {
+		/*
+		 * Proportionately increase size of hover_buffer if the
+		 * non-hover 'donor' buffer is larger than the allocated space.
+		 * It will get scaled down again by wlroots when rendered and as
+		 * required by the current output scale.
+		 *
+		 * This ensures that icons > width or > height keep their aspect
+		 * ratio and are rendered the same as without the hover overlay.
+		 */
+		double scale = MAX((double)icon_width / width,
+				(double)icon_height / height);
+		if (scale > 1.0f) {
+			width = (double)width * scale;
+			height = (double)height * scale;
+		}
+	}
+
+	*hover_buffer = buffer_create_cairo(width, height, 1.0, true);
+
+	cairo_t *cairo = (*hover_buffer)->cairo;
+	cairo_surface_t *surf = cairo_get_target(cairo);
+
+	/* Background */
+	cairo_set_source_surface(cairo, icon.surface,
+		(width - icon_width) / 2, (height - icon_height) / 2);
+	cairo_paint(cairo);
+
+	/* Overlay (non-multiplied alpha) */
+	set_cairo_color(cairo, (float[4]) { 0.5f, 0.5f, 0.5f, 0.3f});
+	cairo_rectangle(cairo, 0, 0, width, height);
+	cairo_fill(cairo);
+	cairo_surface_flush(surf);
+
+	if (icon.is_duplicate) {
+		cairo_surface_destroy(icon.surface);
+	}
+}
+
+/*
+ * We use the following button filename schema: "BUTTON [TOGGLED] [STATE]"
+ * with the words separted by underscore, and the following meaning:
+ *   - BUTTON can be one of 'max', 'iconify', 'close', 'menu'
+ *   - TOGGLED is either 'toggled' or nothing
+ *   - STATE is 'hover' or nothing. In future, 'pressed' may be supported too.
+ *
+ * We believe that this is how the vast majority of extant openbox themes out
+ * there are constructed and it is consistent with the openbox.org wiki. But
+ * please be aware that it is actually different to vanilla Openbox which uses:
+ * "BUTTON [STATE] [TOGGLED]" following an unfortunate commit in 2014 which
+ * broke themes and led to some distros patching Openbox:
+ * https://github.com/danakj/openbox/commit/35e92e4c2a45b28d5c2c9b44b64aeb4222098c94
+ *
+ * Arch Linux and Debian patch Openbox to keep the old syntax (the one we use).
+ * https://gitlab.archlinux.org/archlinux/packaging/packages/openbox/-/blob/main/debian-887908.patch?ref_type=heads
+ * This patch does the following:
+ *   - reads "%s_toggled_pressed.xbm" and "%s_toggled_hover.xbm" instead of the
+ *     'hover_toggled' equivalents.
+ *   - parses 'toggled.unpressed', toggled.pressed' and 'toggled.hover' instead
+ *     of the other way around ('*.toggled') when processing themerc.
+ *
+ * For compatibility with distros which do not apply similar patches, we support
+ * the hover-before-toggle too, for example:
+ *
+ *	.name = "max_toggled_hover",
+ *	.alt_name = "max_hover_toggled",
+ *
+ * ...in the button array definition below.
+ */
 static void
 load_buttons(struct theme *theme)
 {
-	struct button buttons[] = {
-		{
-			"menu",
-			{ 0x00, 0x18, 0x3c, 0x3c, 0x18, 0x00 },
-			{
-				&theme->button_menu_active_unpressed,
-				theme->window_active_button_menu_unpressed_image_color,
-			},
-			{
-				&theme->button_menu_inactive_unpressed,
-				theme->window_inactive_button_menu_unpressed_image_color,
-			},
-		},
-		{
-			"iconify",
-			{ 0x00, 0x00, 0x00, 0x00, 0x3f, 0x3f },
-			{
-				&theme->button_iconify_active_unpressed,
-				theme->window_active_button_iconify_unpressed_image_color,
-			},
-			{
-				&theme->button_iconify_inactive_unpressed,
-				theme->window_inactive_button_iconify_unpressed_image_color,
-			},
-		},
-		{
-			"max",
-			{ 0x3f, 0x3f, 0x21, 0x21, 0x21, 0x3f },
-			{
-				&theme->button_maximize_active_unpressed,
-				theme->window_active_button_max_unpressed_image_color,
-			},
-			{
-				&theme->button_maximize_inactive_unpressed,
-				theme->window_inactive_button_max_unpressed_image_color,
-			},
-		},
-		{
-			"close",
-			{ 0x33, 0x3f, 0x1e, 0x1e, 0x3f, 0x33 },
-			{
-				&theme->button_close_active_unpressed,
-				theme->window_active_button_close_unpressed_image_color,
-			},
-			{
-				&theme->button_close_inactive_unpressed,
-				theme->window_inactive_button_close_unpressed_image_color,
-			},
-		},
-	};
+	struct button buttons[] = { {
+		.name = "menu",
+		.fallback_button = (const char[]){ 0x00, 0x18, 0x3c, 0x3c, 0x18, 0x00 },
+		.active.buffer = &theme->button_menu_active_unpressed,
+		.active.rgba = theme->window_active_button_menu_unpressed_image_color,
+		.inactive.buffer = &theme->button_menu_inactive_unpressed,
+		.inactive.rgba = theme->window_inactive_button_menu_unpressed_image_color,
+	}, {
+		.name = "iconify",
+		.fallback_button = (const char[]){ 0x00, 0x00, 0x00, 0x00, 0x3f, 0x3f },
+		.active.buffer = &theme->button_iconify_active_unpressed,
+		.active.rgba = theme->window_active_button_iconify_unpressed_image_color,
+		.inactive.buffer = &theme->button_iconify_inactive_unpressed,
+		.inactive.rgba = theme->window_inactive_button_iconify_unpressed_image_color,
+	}, {
+		.name = "max",
+		.fallback_button = (const char[]){ 0x3f, 0x3f, 0x21, 0x21, 0x21, 0x3f },
+		.active.buffer = &theme->button_maximize_active_unpressed,
+		.active.rgba = theme->window_active_button_max_unpressed_image_color,
+		.inactive.buffer = &theme->button_maximize_inactive_unpressed,
+		.inactive.rgba = theme->window_inactive_button_max_unpressed_image_color,
+	}, {
+		.name = "max_toggled",
+		.fallback_button = (const char[]){ 0x3e, 0x22, 0x2f, 0x29, 0x39, 0x0f },
+		.active.buffer = &theme->button_restore_active_unpressed,
+		.active.rgba = theme->window_active_button_max_unpressed_image_color,
+		.inactive.buffer = &theme->button_restore_inactive_unpressed,
+		.inactive.rgba = theme->window_inactive_button_max_unpressed_image_color,
+	}, {
+		.name = "close",
+		.fallback_button = (const char[]){ 0x33, 0x3f, 0x1e, 0x1e, 0x3f, 0x33 },
+		.active.buffer = &theme->button_close_active_unpressed,
+		.active.rgba = theme->window_active_button_close_unpressed_image_color,
+		.inactive.buffer = &theme->button_close_inactive_unpressed,
+		.inactive.rgba = theme->window_inactive_button_close_unpressed_image_color,
+	}, {
+		.name = "menu_hover",
+		/* no fallback (non-hover variant is used instead) */
+		.active.buffer = &theme->button_menu_active_hover,
+		.active.rgba = theme->window_active_button_menu_unpressed_image_color,
+		.inactive.buffer = &theme->button_menu_inactive_hover,
+		.inactive.rgba = theme->window_inactive_button_menu_unpressed_image_color,
+	}, {
+		.name = "iconify_hover",
+		/* no fallback (non-hover variant is used instead) */
+		.active.buffer = &theme->button_iconify_active_hover,
+		.active.rgba = theme->window_active_button_iconify_unpressed_image_color,
+		.inactive.buffer = &theme->button_iconify_inactive_hover,
+		.inactive.rgba = theme->window_inactive_button_iconify_unpressed_image_color,
+	}, {
+		.name = "max_hover",
+		/* no fallback (non-hover variant is used instead) */
+		.active.buffer = &theme->button_maximize_active_hover,
+		.active.rgba = theme->window_active_button_max_unpressed_image_color,
+		.inactive.buffer = &theme->button_maximize_inactive_hover,
+		.inactive.rgba = theme->window_inactive_button_max_unpressed_image_color,
+	}, {
+		.name = "max_toggled_hover",
+		.alt_name = "max_hover_toggled",
+		/* no fallback (non-hover variant is used instead) */
+		.active.buffer = &theme->button_restore_active_hover,
+		.active.rgba = theme->window_active_button_max_unpressed_image_color,
+		.inactive.buffer = &theme->button_restore_inactive_hover,
+		.inactive.rgba = theme->window_inactive_button_max_unpressed_image_color,
+	}, {
+		.name = "close_hover",
+		/* no fallback (non-hover variant is used instead) */
+		.active.buffer = &theme->button_close_active_hover,
+		.active.rgba = theme->window_active_button_close_unpressed_image_color,
+		.inactive.buffer = &theme->button_close_inactive_hover,
+		.inactive.rgba = theme->window_inactive_button_close_unpressed_image_color,
+	}, };
 
 	char filename[4096] = {0};
 	for (size_t i = 0; i < ARRAY_SIZE(buttons); ++i) {
@@ -116,14 +225,14 @@ load_buttons(struct theme *theme)
 		drop(b->active.buffer);
 		drop(b->inactive.buffer);
 
-		/* Try png icon first */
+		/* PNG */
 		snprintf(filename, sizeof(filename), "%s-active.png", b->name);
 		button_png_load(filename, b->active.buffer);
 		snprintf(filename, sizeof(filename), "%s-inactive.png", b->name);
 		button_png_load(filename, b->inactive.buffer);
 
 #if HAVE_RSVG
-		/* Then try svg icon */
+		/* SVG */
 		int size = theme->title_height - 2 * theme->padding_height;
 		if (!*b->active.buffer) {
 			snprintf(filename, sizeof(filename), "%s-active.svg", b->name);
@@ -135,15 +244,80 @@ load_buttons(struct theme *theme)
 		}
 #endif
 
-		/* If there were no png/svg buttons, use xbm */
+		/* XBM */
 		snprintf(filename, sizeof(filename), "%s.xbm", b->name);
 		if (!*b->active.buffer) {
-			button_xbm_load(filename, b->active.buffer,
-				b->fallback_button, b->active.rgba);
+			button_xbm_load(filename, b->active.buffer, b->active.rgba);
 		}
 		if (!*b->inactive.buffer) {
-			button_xbm_load(filename, b->inactive.buffer,
-				b->fallback_button, b->inactive.rgba);
+			button_xbm_load(filename, b->inactive.buffer, b->inactive.rgba);
+		}
+
+		/*
+		 * XBM (alternative name)
+		 * For example max_hover_toggled instead of max_toggled_hover
+		 */
+		if (b->alt_name) {
+			snprintf(filename, sizeof(filename), "%s.xbm", b->name);
+		}  else {
+			filename[0] = '\0';
+		}
+		if (!*b->active.buffer) {
+			button_xbm_load(filename, b->active.buffer, b->active.rgba);
+		}
+		if (!*b->inactive.buffer) {
+			button_xbm_load(filename, b->inactive.buffer, b->inactive.rgba);
+		}
+
+		/*
+		 * Builtin bitmap
+		 *
+		 * Applicable to basic buttons such as max, max_toggled and
+		 * iconify. There are no bitmap fallbacks for *_hover icons.
+		 */
+		if (!b->fallback_button) {
+			continue;
+		}
+		if (!*b->active.buffer) {
+			button_xbm_from_bitmap(b->fallback_button,
+				b->active.buffer, b->active.rgba);
+		}
+		if (!*b->inactive.buffer) {
+			button_xbm_from_bitmap(b->fallback_button,
+				b->inactive.buffer, b->inactive.rgba);
+		}
+	}
+
+	/*
+	 * If hover-icons do not exist, add fallbacks by coping the non-hover
+	 * variant (base) and then adding an overlay.
+	 */
+	for (size_t i = 0; i < ARRAY_SIZE(buttons); i++) {
+		struct button *hover_button = &buttons[i];
+
+		if (!strstr(hover_button->name, "_hover")) {
+			continue;
+		}
+
+		/* If name=='foo_hover', basename='foo' */
+		char basename[64]  = {0};
+		snprintf(basename, sizeof(basename), "%s", hover_button->name);
+		trim_last_field(basename, '_');
+		for (size_t j = 0; j < ARRAY_SIZE(buttons); j++) {
+			struct button *base = &buttons[j];
+			if (!strcmp(basename, base->name)) {
+				if (!*hover_button->active.buffer) {
+					create_hover_fallback(theme,
+						hover_button->active.buffer,
+						*base->active.buffer);
+				}
+				if (!*hover_button->inactive.buffer) {
+					create_hover_fallback(theme,
+						hover_button->inactive.buffer,
+						*base->inactive.buffer);
+				}
+				break;
+			}
 		}
 	}
 }
@@ -217,12 +391,12 @@ theme_builtin(struct theme *theme)
 	theme->menu_overlap_x = 0;
 	theme->menu_overlap_y = 0;
 
-	parse_hexstr("#dddad6", theme->window_active_border_color);
+	parse_hexstr("#e1dedb", theme->window_active_border_color);
 	parse_hexstr("#f6f5f4", theme->window_inactive_border_color);
 
 	parse_hexstr("#ff0000", theme->window_toggled_keybinds_color);
 
-	parse_hexstr("#dddad6", theme->window_active_title_bg_color);
+	parse_hexstr("#e1dedb", theme->window_active_title_bg_color);
 	parse_hexstr("#f6f5f4", theme->window_inactive_title_bg_color);
 
 	parse_hexstr("#000000", theme->window_active_label_text_color);
@@ -248,7 +422,7 @@ theme_builtin(struct theme *theme)
 
 	parse_hexstr("#fcfbfa", theme->menu_items_bg_color);
 	parse_hexstr("#000000", theme->menu_items_text_color);
-	parse_hexstr("#dddad6", theme->menu_items_active_bg_color);
+	parse_hexstr("#e1dedb", theme->menu_items_active_bg_color);
 	parse_hexstr("#000000", theme->menu_items_active_text_color);
 
 	theme->menu_item_padding_x = 7;
@@ -268,6 +442,9 @@ theme_builtin(struct theme *theme)
 	theme->osd_window_switcher_item_padding_y = 1;
 	theme->osd_window_switcher_item_active_border_width = 2;
 
+	theme->osd_workspace_switcher_boxes_width = 20;
+	theme->osd_workspace_switcher_boxes_height = 20;
+
 	/* inherit settings in post_processing() if not set elsewhere */
 	theme->osd_bg_color[0] = FLT_MIN;
 	theme->osd_border_width = INT_MIN;
@@ -454,6 +631,12 @@ entry(struct theme *theme, const char *key, const char *value)
 	if (match_glob(key, "osd.window-switcher.item.active.border.width")) {
 		theme->osd_window_switcher_item_active_border_width = atoi(value);
 	}
+	if (match_glob(key, "osd.workspace-switcher.boxes.width")) {
+		theme->osd_workspace_switcher_boxes_width = atoi(value);
+	}
+	if (match_glob(key, "osd.workspace-switcher.boxes.height")) {
+		theme->osd_workspace_switcher_boxes_height = atoi(value);
+	}
 	if (match_glob(key, "osd.label.text.color")) {
 		parse_hexstr(value, theme->osd_label_text_color);
 	}
@@ -754,7 +937,7 @@ create_corners(struct theme *theme)
 static void
 post_processing(struct theme *theme)
 {
-	int h = font_height(&rc.font_activewindow);
+	int h = MAX(font_height(&rc.font_activewindow), font_height(&rc.font_inactivewindow));
 	if (theme->title_height < h) {
 		theme->title_height = h + 2 * theme->padding_height;
 	}
@@ -802,6 +985,12 @@ post_processing(struct theme *theme)
 		memcpy(theme->osd_border_color, theme->osd_label_text_color,
 			sizeof(theme->osd_border_color));
 	}
+	if (theme->osd_workspace_switcher_boxes_width == 0) {
+		theme->osd_workspace_switcher_boxes_height = 0;
+	}
+	if (theme->osd_workspace_switcher_boxes_height == 0) {
+		theme->osd_workspace_switcher_boxes_width = 0;
+	}
 }
 
 void
