@@ -34,20 +34,16 @@ view_from_wlr_surface(struct wlr_surface *surface)
 	 * - find a way to get rid of xdg/xwayland-specific stuff
 	 * - look up root/toplevel surface if passed a subsurface?
 	 */
-	if (wlr_surface_is_xdg_surface(surface)) {
-		struct wlr_xdg_surface *xdg_surface =
-			wlr_xdg_surface_from_wlr_surface(surface);
-		if (xdg_surface) {
-			return xdg_surface->data;
-		}
+	struct wlr_xdg_surface *xdg_surface =
+		wlr_xdg_surface_try_from_wlr_surface(surface);
+	if (xdg_surface) {
+		return xdg_surface->data;
 	}
 #if HAVE_XWAYLAND
-	if (wlr_surface_is_xwayland_surface(surface)) {
-		struct wlr_xwayland_surface *xsurface =
-			wlr_xwayland_surface_from_wlr_surface(surface);
-		if (xsurface) {
-			return xsurface->data;
-		}
+	struct wlr_xwayland_surface *xsurface =
+		wlr_xwayland_surface_try_from_wlr_surface(surface);
+	if (xsurface) {
+		return xsurface->data;
 	}
 #endif
 	return NULL;
@@ -597,7 +593,9 @@ view_adjust_floating_geometry(struct view *view, struct wlr_box *geometry)
 		return false;
 	}
 
-	if (window_rules_get_property(view, "fixedPosition") == LAB_PROP_TRUE) {
+	/* Avoid moving panels out of their own reserved area ("strut") */
+	if (window_rules_get_property(view, "fixedPosition") == LAB_PROP_TRUE
+			|| view_has_strut_partial(view)) {
 		return false;
 	}
 
@@ -1049,6 +1047,13 @@ view_toggle_always_on_bottom(struct view *view)
 	}
 }
 
+void
+view_toggle_visible_on_all_workspaces(struct view *view)
+{
+	assert(view);
+	view->visible_on_all_workspaces = !view->visible_on_all_workspaces;
+}
+
 void
 view_move_to_workspace(struct view *view, struct workspace *workspace)
 {
@@ -1066,7 +1071,7 @@ decorate(struct view *view)
 {
 	if (!view->ssd) {
 		view->ssd = ssd_create(view,
-			view == view->server->focused_view);
+			view == view->server->active_view);
 	}
 }
 
@@ -1224,6 +1229,56 @@ view_on_output_destroy(struct view *view)
 	view->output = NULL;
 }
 
+static struct output *
+view_get_adjacent_output(struct view *view, enum view_edge edge)
+{
+	assert(view);
+	struct output *output = view->output;
+	if (!output_is_usable(output)) {
+		wlr_log(WLR_ERROR,
+			"view has no output, cannot find adjacent output");
+		return NULL;
+	}
+
+	/* Determine any adjacent output in the appropriate direction */
+	struct wlr_output *new_output = NULL;
+	struct wlr_output *current_output = output->wlr_output;
+	struct wlr_output_layout *layout = view->server->output_layout;
+	switch (edge) {
+	case VIEW_EDGE_LEFT:
+		new_output = wlr_output_layout_adjacent_output(
+			layout, WLR_DIRECTION_LEFT, current_output, 1, 0);
+		break;
+	case VIEW_EDGE_RIGHT:
+		new_output = wlr_output_layout_adjacent_output(
+			layout, WLR_DIRECTION_RIGHT, current_output, 1, 0);
+		break;
+	case VIEW_EDGE_UP:
+		new_output = wlr_output_layout_adjacent_output(
+			layout, WLR_DIRECTION_UP, current_output, 0, 1);
+		break;
+	case VIEW_EDGE_DOWN:
+		new_output = wlr_output_layout_adjacent_output(
+			layout, WLR_DIRECTION_DOWN, current_output, 0, 1);
+		break;
+	default:
+		break;
+	}
+
+	/* When "adjacent" output is the same as the original, there is no adjacent */
+	if (!new_output || new_output == current_output) {
+		return NULL;
+	}
+
+	output = output_from_wlr_output(view->server, new_output);
+	if (!output_is_usable(output)) {
+		wlr_log(WLR_ERROR, "invalid output in layout");
+		return NULL;
+	}
+
+	return output;
+}
+
 void
 view_move_to_edge(struct view *view, enum view_edge direction, bool snap_to_windows)
 {
@@ -1256,7 +1311,78 @@ view_move_to_edge(struct view *view, enum view_edge direction, bool snap_to_wind
 		}
 	}
 
-	view_move(view, view->pending.x + dx, view->pending.y + dy);
+	if (dx != 0 || dy != 0) {
+		/* Move the window if a change was discovered */
+		view_move(view, view->pending.x + dx, view->pending.y + dy);
+		return;
+	}
+
+	/* If the view is maximized, do not attempt to jump displays */
+	if (view->maximized != VIEW_AXIS_NONE) {
+		return;
+	}
+
+	/* Otherwise, move to edge of next adjacent display, if possible */
+	struct output *output = view_get_adjacent_output(view, direction);
+	if (!output) {
+		return;
+	}
+
+	/* When jumping to next output, attach to edge nearest the motion */
+	struct wlr_box usable = output_usable_area_in_layout_coords(output);
+	struct border margin = ssd_get_margin(view->ssd);
+
+	/* Bounds of the possible placement zone in this output */
+	int left = usable.x + rc.gap + margin.left;
+	int right = usable.x + usable.width - rc.gap - margin.right;
+	int top = usable.y + rc.gap + margin.top;
+	int bottom = usable.y + usable.height - rc.gap - margin.bottom;
+
+	/* Default target position on new output is current target position */
+	int destination_x = view->pending.x;
+	int destination_y = view->pending.y;
+
+	/* Compute the new position in the direction of motion */
+	direction = view_edge_invert(direction);
+	switch (direction) {
+	case VIEW_EDGE_LEFT:
+		destination_x = left;
+		break;
+	case VIEW_EDGE_RIGHT:
+		destination_x = right - view->pending.width;
+		break;
+	case VIEW_EDGE_UP:
+		destination_y = top;
+		break;
+	case VIEW_EDGE_DOWN:
+		destination_y = bottom - view->pending.height;
+		break;
+	default:
+		return;
+	}
+
+	/* If more than half the view is right of usable region, align to right */
+	int midpoint = destination_x + view->pending.width / 2;
+
+	if (destination_x >= left && midpoint > usable.x + usable.width) {
+		destination_x = right - view->pending.width;
+	}
+
+	/* Never allow the window to start left of the usable edge */
+	destination_x = MAX(destination_x, left);
+
+	/* If more than half the view is below usable region, align to bottom */
+	midpoint = destination_y + view->pending.height / 2;
+	if (destination_y >= top && midpoint > usable.y + usable.height) {
+		destination_y = bottom - view->pending.height;
+	}
+
+	/* Never allow the window to start above the usable edge */
+	destination_y = MAX(destination_y, top);
+
+	view_set_untiled(view);
+	view_set_output(view, output);
+	view_move(view, destination_x, destination_y);
 }
 
 void
@@ -1348,39 +1474,10 @@ view_snap_to_edge(struct view *view, enum view_edge edge,
 	}
 
 	if (across_outputs && view->tiled == edge && view->maximized == VIEW_AXIS_NONE) {
-		/* We are already tiled for this edge and thus should switch outputs */
-		struct wlr_output *new_output = NULL;
-		struct wlr_output *current_output = output->wlr_output;
-		struct wlr_output_layout *layout = view->server->output_layout;
-		switch (edge) {
-		case VIEW_EDGE_LEFT:
-			new_output = wlr_output_layout_adjacent_output(
-				layout, WLR_DIRECTION_LEFT, current_output, 1, 0);
-			break;
-		case VIEW_EDGE_RIGHT:
-			new_output = wlr_output_layout_adjacent_output(
-				layout, WLR_DIRECTION_RIGHT, current_output, 1, 0);
-			break;
-		case VIEW_EDGE_UP:
-			new_output = wlr_output_layout_adjacent_output(
-				layout, WLR_DIRECTION_UP, current_output, 0, 1);
-			break;
-		case VIEW_EDGE_DOWN:
-			new_output = wlr_output_layout_adjacent_output(
-				layout, WLR_DIRECTION_DOWN, current_output, 0, 1);
-			break;
-		default:
-			break;
-		}
-		if (new_output && new_output != current_output) {
-			/* Move to next output */
-			edge = view_edge_invert(edge);
-			output = output_from_wlr_output(view->server, new_output);
-			if (!output_is_usable(output)) {
-				wlr_log(WLR_ERROR, "invalid output in layout");
-				return;
-			}
-		} else {
+		/* We are already tiled for this edge; try to switch outputs */
+		output = view_get_adjacent_output(view, edge);
+
+		if (!output) {
 			/*
 			 * No more output to move to
 			 *
@@ -1392,6 +1489,9 @@ view_snap_to_edge(struct view *view, enum view_edge edge,
 			view_apply_tiled_geometry(view);
 			return;
 		}
+
+		/* When switching outputs, jump to the opposite edge */
+		edge = view_edge_invert(edge);
 	}
 
 	if (view->maximized != VIEW_AXIS_NONE) {
@@ -1547,6 +1647,14 @@ view_is_related(struct view *view, struct wlr_surface *surface)
 	return false;
 }
 
+bool
+view_has_strut_partial(struct view *view)
+{
+	assert(view);
+	return view->impl->has_strut_partial &&
+		view->impl->has_strut_partial(view);
+}
+
 const char *
 view_get_string_prop(struct view *view, const char *prop)
 {
@@ -1609,6 +1717,55 @@ view_toggle_keybinds(struct view *view)
 	}
 }
 
+void
+mappable_connect(struct mappable *mappable, struct wlr_surface *surface,
+		wl_notify_func_t notify_map, wl_notify_func_t notify_unmap)
+{
+	assert(mappable);
+	assert(!mappable->connected);
+	mappable->map.notify = notify_map;
+	wl_signal_add(&surface->events.map, &mappable->map);
+	mappable->unmap.notify = notify_unmap;
+	wl_signal_add(&surface->events.unmap, &mappable->unmap);
+	mappable->connected = true;
+}
+
+void
+mappable_disconnect(struct mappable *mappable)
+{
+	assert(mappable);
+	assert(mappable->connected);
+	wl_list_remove(&mappable->map.link);
+	wl_list_remove(&mappable->unmap.link);
+	mappable->connected = false;
+}
+
+static void
+handle_map(struct wl_listener *listener, void *data)
+{
+	struct view *view = wl_container_of(listener, view, mappable.map);
+	view->impl->map(view);
+}
+
+static void
+handle_unmap(struct wl_listener *listener, void *data)
+{
+	struct view *view = wl_container_of(listener, view, mappable.unmap);
+	view->impl->unmap(view, /* client_request */ true);
+}
+
+/*
+ * TODO: after the release of wlroots 0.17, consider incorporating this
+ * function into a more general view_set_surface() function, which could
+ * connect other surface event handlers (like commit) as well.
+ */
+void
+view_connect_map(struct view *view, struct wlr_surface *surface)
+{
+	assert(view);
+	mappable_connect(&view->mappable, surface, handle_map, handle_unmap);
+}
+
 void
 view_destroy(struct view *view)
 {
@@ -1616,8 +1773,10 @@ view_destroy(struct view *view)
 	struct server *server = view->server;
 	bool need_cursor_update = false;
 
-	wl_list_remove(&view->map.link);
-	wl_list_remove(&view->unmap.link);
+	if (view->mappable.connected) {
+		mappable_disconnect(&view->mappable);
+	}
+
 	wl_list_remove(&view->request_move.link);
 	wl_list_remove(&view->request_resize.link);
 	wl_list_remove(&view->request_minimize.link);
@@ -1638,8 +1797,8 @@ view_destroy(struct view *view)
 		regions_hide_overlay(&server->seat);
 	}
 
-	if (server->focused_view == view) {
-		server->focused_view = NULL;
+	if (server->active_view == view) {
+		server->active_view = NULL;
 		need_cursor_update = true;
 	}
 
