@@ -128,6 +128,10 @@ window_rules_get_property(struct view *view, const char *property)
 					&& !strcasecmp(property, "ignoreFocusRequest")) {
 				return rule->ignore_focus_request;
 			}
+			if (rule->fixed_position
+					&& !strcasecmp(property, "fixedPosition")) {
+				return rule->fixed_position;
+			}
 		}
 	}
 	return LAB_PROP_UNSPECIFIED;
