@@ -163,20 +163,6 @@ set_pending_configure_serial(struct view *view, uint32_t serial)
 		CONFIGURE_TIMEOUT_MS);
 }
 
-static void
-handle_map(struct wl_listener *listener, void *data)
-{
-	struct view *view = wl_container_of(listener, view, map);
-	view->impl->map(view);
-}
-
-static void
-handle_unmap(struct wl_listener *listener, void *data)
-{
-	struct view *view = wl_container_of(listener, view, unmap);
-	view->impl->unmap(view, /* client_request */ true);
-}
-
 static void
 handle_destroy(struct wl_listener *listener, void *data)
 {
@@ -579,12 +565,12 @@ xdg_activation_handle_request(struct wl_listener *listener, void *data)
 {
 	const struct wlr_xdg_activation_v1_request_activate_event *event = data;
 
-	if (!wlr_surface_is_xdg_surface(event->surface)) {
+	struct wlr_xdg_surface *xdg_surface =
+		wlr_xdg_surface_try_from_wlr_surface(event->surface);
+	if (!xdg_surface) {
 		return;
 	}
-	struct wlr_xdg_surface *xdg_surface =
-		wlr_xdg_surface_from_wlr_surface(event->surface);
-	struct view *view = xdg_surface ? xdg_surface->data : NULL;
+	struct view *view = xdg_surface->data;
 
 	if (!view) {
 		wlr_log(WLR_INFO, "Not activating surface - no view attached to surface");
@@ -688,8 +674,7 @@ xdg_surface_new(struct wl_listener *listener, void *data)
 	/* In support of xdg popups */
 	xdg_surface->surface->data = tree;
 
-	CONNECT_SIGNAL(xdg_surface, view, map);
-	CONNECT_SIGNAL(xdg_surface, view, unmap);
+	view_connect_map(view, xdg_surface->surface);
 	CONNECT_SIGNAL(xdg_surface, view, destroy);
 
 	struct wlr_xdg_toplevel *toplevel = xdg_surface->toplevel;
