@@ -15,6 +15,8 @@
 #include "workspaces.h"
 #include "xwayland.h"
 
+static void xwayland_view_unmap(struct view *view, bool client_request);
+
 static struct view_size_hints
 xwayland_view_get_size_hints(struct view *view)
 {
@@ -101,6 +103,14 @@ xwayland_view_wants_focus(struct view *view)
 	return VIEW_WANTS_FOCUS_NEVER;
 }
 
+static bool
+xwayland_view_has_strut_partial(struct view *view)
+{
+	struct wlr_xwayland_surface *xsurface =
+		xwayland_surface_from_view(view);
+	return (bool)xsurface->strut_partial;
+}
+
 static struct wlr_xwayland_surface *
 top_parent_of(struct view *view)
 {
@@ -225,17 +235,24 @@ handle_request_resize(struct wl_listener *listener, void *data)
 }
 
 static void
-handle_map(struct wl_listener *listener, void *data)
+handle_associate(struct wl_listener *listener, void *data)
 {
-	struct view *view = wl_container_of(listener, view, map);
-	view->impl->map(view);
+	struct xwayland_view *xwayland_view =
+		wl_container_of(listener, xwayland_view, associate);
+	assert(xwayland_view->xwayland_surface &&
+		xwayland_view->xwayland_surface->surface);
+
+	view_connect_map(&xwayland_view->base,
+		xwayland_view->xwayland_surface->surface);
 }
 
 static void
-handle_unmap(struct wl_listener *listener, void *data)
+handle_dissociate(struct wl_listener *listener, void *data)
 {
-	struct view *view = wl_container_of(listener, view, unmap);
-	view->impl->unmap(view, /* client_request */ true);
+	struct xwayland_view *xwayland_view =
+		wl_container_of(listener, xwayland_view, dissociate);
+
+	mappable_disconnect(&xwayland_view->base.mappable);
 }
 
 static void
@@ -275,11 +292,14 @@ handle_destroy(struct wl_listener *listener, void *data)
 	xwayland_view->xwayland_surface = NULL;
 
 	/* Remove XWayland view specific listeners */
+	wl_list_remove(&xwayland_view->associate.link);
+	wl_list_remove(&xwayland_view->dissociate.link);
 	wl_list_remove(&xwayland_view->request_activate.link);
 	wl_list_remove(&xwayland_view->request_configure.link);
 	wl_list_remove(&xwayland_view->set_class.link);
 	wl_list_remove(&xwayland_view->set_decorations.link);
 	wl_list_remove(&xwayland_view->set_override_redirect.link);
+	wl_list_remove(&xwayland_view->set_strut_partial.link);
 
 	view_destroy(view);
 }
@@ -291,9 +311,20 @@ xwayland_view_configure(struct view *view, struct wlr_box geo)
 	wlr_xwayland_surface_configure(xwayland_surface_from_view(view),
 		geo.x, geo.y, geo.width, geo.height);
 
+	/*
+	 * For unknown reasons, XWayland surfaces that are completely
+	 * offscreen seem not to generate commit events. In rare cases,
+	 * this can prevent an offscreen window from moving onscreen
+	 * (since we wait for a commit event that never occurs). As a
+	 * workaround, move offscreen surfaces immediately.
+	 */
+	bool is_offscreen = !wlr_box_empty(&view->current) &&
+		!wlr_output_layout_intersects(view->server->output_layout, NULL,
+			&view->current);
+
 	/* If not resizing, process the move immediately */
-	if (view->current.width == geo.width
-			&& view->current.height == geo.height) {
+	if (is_offscreen || (view->current.width == geo.width
+			&& view->current.height == geo.height)) {
 		view->current.x = geo.x;
 		view->current.y = geo.y;
 		view_moved(view);
@@ -308,12 +339,21 @@ handle_request_configure(struct wl_listener *listener, void *data)
 	struct view *view = &xwayland_view->base;
 	struct wlr_xwayland_surface_configure_event *event = data;
 
-	int width = event->width;
-	int height = event->height;
-	view_adjust_size(view, &width, &height);
-
-	xwayland_view_configure(view,
-		(struct wlr_box){event->x, event->y, width, height});
+	if (view_is_floating(view)) {
+		/* Honor client configure requests for floating views */
+		struct wlr_box box = {.x = event->x, .y = event->y,
+			.width = event->width, .height = event->height};
+		view_adjust_size(view, &box.width, &box.height);
+		xwayland_view_configure(view, box);
+	} else {
+		/*
+		 * Do not allow clients to request geometry other than
+		 * what we computed for maximized/fullscreen/tiled
+		 * views. Ignore the client request and send back a
+		 * ConfigureNotify event with the computed geometry.
+		 */
+		xwayland_view_configure(view, view->pending);
+	}
 }
 
 static void
@@ -434,15 +474,27 @@ handle_set_override_redirect(struct wl_listener *listener, void *data)
 	struct wlr_xwayland_surface *xsurface = xwayland_view->xwayland_surface;
 
 	struct server *server = view->server;
-	bool mapped = xsurface->mapped;
+	bool mapped = xsurface->surface && xsurface->surface->mapped;
 	if (mapped) {
-		handle_unmap(&view->unmap, xsurface);
+		xwayland_view_unmap(view, /* client_request */ true);
 	}
 	handle_destroy(&view->destroy, xsurface);
 	/* view is invalid after this point */
 	xwayland_unmanaged_create(server, xsurface, mapped);
 }
 
+static void
+handle_set_strut_partial(struct wl_listener *listener, void *data)
+{
+	struct xwayland_view *xwayland_view =
+		wl_container_of(listener, xwayland_view, set_strut_partial);
+	struct view *view = &xwayland_view->base;
+
+	if (view->mapped) {
+		output_update_all_usable_areas(view->server, false);
+	}
+}
+
 static void
 set_initial_position(struct view *view,
 		struct wlr_xwayland_surface *xwayland_surface)
@@ -491,7 +543,7 @@ init_foreign_toplevel(struct view *view)
 		return;
 	}
 	struct view *parent = (struct view *)surface->parent->data;
-	if (!parent->toplevel.handle) {
+	if (!parent || !parent->toplevel.handle) {
 		return;
 	}
 	wlr_foreign_toplevel_handle_v1_set_parent(view->toplevel.handle, parent->toplevel.handle);
@@ -585,6 +637,11 @@ xwayland_view_map(struct view *view)
 
 	view_impl_map(view);
 	view->been_mapped = true;
+
+	/* Update usable area to account for XWayland "struts" (panels) */
+	if (xwayland_surface->strut_partial) {
+		output_update_all_usable_areas(view->server, false);
+	}
 }
 
 static void
@@ -598,6 +655,11 @@ xwayland_view_unmap(struct view *view, bool client_request)
 	wlr_scene_node_set_enabled(&view->scene_tree->node, false);
 	view_impl_unmap(view);
 
+	/* Update usable area to account for XWayland "struts" (panels) */
+	if (xwayland_surface_from_view(view)->strut_partial) {
+		output_update_all_usable_areas(view->server, false);
+	}
+
 	/*
 	 * If the view was explicitly unmapped by the client (rather
 	 * than just minimized), destroy the foreign toplevel handle so
@@ -701,8 +763,7 @@ xwayland_view_is_related(struct view *view, struct wlr_surface *surface)
 	struct wlr_xwayland_surface *xsurface =
 		xwayland_surface_from_view(view);
 	struct wlr_xwayland_surface *xsurface2 =
-		wlr_surface_is_xwayland_surface(surface) ?
-		wlr_xwayland_surface_from_wlr_surface(surface) : NULL;
+		wlr_xwayland_surface_try_from_wlr_surface(surface);
 
 	return (xsurface2 && xsurface2->pid == xsurface->pid);
 }
@@ -744,6 +805,7 @@ static const struct view_impl xwayland_view_impl = {
 	.is_related = xwayland_view_is_related,
 	.get_size_hints = xwayland_view_get_size_hints,
 	.wants_focus = xwayland_view_wants_focus,
+	.has_strut_partial = xwayland_view_has_strut_partial,
 };
 
 void
@@ -771,8 +833,6 @@ xwayland_view_create(struct server *server,
 	view->scene_tree = wlr_scene_tree_create(view->workspace->tree);
 	node_descriptor_create(&view->scene_tree->node, LAB_NODE_DESC_VIEW, view);
 
-	CONNECT_SIGNAL(xsurface, view, map);
-	CONNECT_SIGNAL(xsurface, view, unmap);
 	CONNECT_SIGNAL(xsurface, view, destroy);
 	CONNECT_SIGNAL(xsurface, view, request_minimize);
 	CONNECT_SIGNAL(xsurface, view, request_maximize);
@@ -782,11 +842,14 @@ xwayland_view_create(struct server *server,
 	CONNECT_SIGNAL(xsurface, view, set_title);
 
 	/* Events specific to XWayland views */
+	CONNECT_SIGNAL(xsurface, xwayland_view, associate);
+	CONNECT_SIGNAL(xsurface, xwayland_view, dissociate);
 	CONNECT_SIGNAL(xsurface, xwayland_view, request_activate);
 	CONNECT_SIGNAL(xsurface, xwayland_view, request_configure);
 	CONNECT_SIGNAL(xsurface, xwayland_view, set_class);
 	CONNECT_SIGNAL(xsurface, xwayland_view, set_decorations);
 	CONNECT_SIGNAL(xsurface, xwayland_view, set_override_redirect);
+	CONNECT_SIGNAL(xsurface, xwayland_view, set_strut_partial);
 
 	wl_list_insert(&view->server->views, &view->link);
 
@@ -820,6 +883,7 @@ handle_ready(struct wl_listener *listener, void *data)
 	struct server *server =
 		wl_container_of(listener, server, xwayland_ready);
 	wlr_xwayland_set_seat(server->xwayland, server->seat.seat);
+	xwayland_update_workarea(server);
 }
 
 void
@@ -909,3 +973,170 @@ xwayland_server_finish(struct server *server)
 	server->xwayland = NULL;
 	wlr_xwayland_destroy(xwayland);
 }
+
+static bool
+intervals_overlap(int start_a, int end_a, int start_b, int end_b)
+{
+	/* check for empty intervals */
+	if (end_a <= start_a || end_b <= start_b) {
+		return false;
+	}
+
+	return start_a < start_b ?
+		start_b < end_a :  /* B starts within A */
+		start_a < end_b;   /* A starts within B */
+}
+
+/*
+ * Subtract the area of an XWayland view (e.g. panel) from the usable
+ * area of the output based on _NET_WM_STRUT_PARTIAL property.
+ */
+void
+xwayland_adjust_usable_area(struct view *view, struct wlr_output_layout *layout,
+		struct wlr_output *output, struct wlr_box *usable)
+{
+	assert(view);
+	assert(layout);
+	assert(output);
+	assert(usable);
+
+	if (view->type != LAB_XWAYLAND_VIEW) {
+		return;
+	}
+
+	xcb_ewmh_wm_strut_partial_t *strut =
+		xwayland_surface_from_view(view)->strut_partial;
+	if (!strut) {
+		return;
+	}
+
+	/* these are layout coordinates */
+	struct wlr_box lb = { 0 };
+	wlr_output_layout_get_box(layout, NULL, &lb);
+	struct wlr_box ob = { 0 };
+	wlr_output_layout_get_box(layout, output, &ob);
+
+	/*
+	 * strut->right/bottom are offsets from the lower right corner
+	 * of the X11 screen, which should generally correspond with the
+	 * lower right corner of the output layout
+	 */
+	double strut_left = strut->left;
+	double strut_right = (lb.x + lb.width) - strut->right;
+	double strut_top = strut->top;
+	double strut_bottom = (lb.y + lb.height) - strut->bottom;
+
+	/* convert layout to output coordinates */
+	wlr_output_layout_output_coords(layout, output,
+		&strut_left, &strut_top);
+	wlr_output_layout_output_coords(layout, output,
+		&strut_right, &strut_bottom);
+
+	/* deal with right/bottom rather than width/height */
+	int usable_right = usable->x + usable->width;
+	int usable_bottom = usable->y + usable->height;
+
+	/* here we mix output and layout coordinates; be careful */
+	if (strut_left > usable->x && strut_left < usable_right
+			&& intervals_overlap(ob.y, ob.y + ob.height,
+			strut->left_start_y, strut->left_end_y + 1)) {
+		usable->x = strut_left;
+	}
+	if (strut_right > usable->x && strut_right < usable_right
+			&& intervals_overlap(ob.y, ob.y + ob.height,
+			strut->right_start_y, strut->right_end_y + 1)) {
+		usable_right = strut_right;
+	}
+	if (strut_top > usable->y && strut_top < usable_bottom
+			&& intervals_overlap(ob.x, ob.x + ob.width,
+			strut->top_start_x, strut->top_end_x + 1)) {
+		usable->y = strut_top;
+	}
+	if (strut_bottom > usable->y && strut_bottom < usable_bottom
+			&& intervals_overlap(ob.x, ob.x + ob.width,
+			strut->bottom_start_x, strut->bottom_end_x + 1)) {
+		usable_bottom = strut_bottom;
+	}
+
+	usable->width = usable_right - usable->x;
+	usable->height = usable_bottom - usable->y;
+}
+
+void
+xwayland_update_workarea(struct server *server)
+{
+	/*
+	 * Do nothing if called during destroy or before xwayland is ready.
+	 * This function will be called again from the ready signal handler.
+	 */
+	if (!server->xwayland || !server->xwayland->xwm) {
+		return;
+	}
+
+	struct wlr_box lb;
+	wlr_output_layout_get_box(server->output_layout, NULL, &lb);
+
+	/* Compute outer edges of layout (excluding negative regions) */
+	int layout_left = MAX(0, lb.x);
+	int layout_right = MAX(0, lb.x + lb.width);
+	int layout_top = MAX(0, lb.y);
+	int layout_bottom = MAX(0, lb.y + lb.height);
+
+	/* Workarea is initially the entire layout */
+	int workarea_left = layout_left;
+	int workarea_right = layout_right;
+	int workarea_top = layout_top;
+	int workarea_bottom = layout_bottom;
+
+	struct output *output;
+	wl_list_for_each(output, &server->outputs, link) {
+		if (!output_is_usable(output)) {
+			continue;
+		}
+
+		struct wlr_box ob;
+		wlr_output_layout_get_box(server->output_layout,
+			output->wlr_output, &ob);
+
+		/* Compute edges of output */
+		int output_left = ob.x;
+		int output_right = ob.x + ob.width;
+		int output_top = ob.y;
+		int output_bottom = ob.y + ob.height;
+
+		/* Compute edges of usable area */
+		int usable_left = output_left + output->usable_area.x;
+		int usable_right = usable_left + output->usable_area.width;
+		int usable_top = output_top + output->usable_area.y;
+		int usable_bottom = usable_top + output->usable_area.height;
+
+		/*
+		 * Only adjust workarea edges for output edges that are
+		 * aligned with outer edges of layout
+		 */
+		if (output_left == layout_left) {
+			workarea_left = MAX(workarea_left, usable_left);
+		}
+		if (output_right == layout_right) {
+			workarea_right = MIN(workarea_right, usable_right);
+		}
+		if (output_top == layout_top) {
+			workarea_top = MAX(workarea_top, usable_top);
+		}
+		if (output_bottom == layout_bottom) {
+			workarea_bottom = MIN(workarea_bottom, usable_bottom);
+		}
+	}
+
+	/*
+	 * Set _NET_WORKAREA property. We don't report virtual desktops
+	 * to XWayland, so we set only one workarea.
+	 */
+	struct wlr_box workarea = {
+		.x = workarea_left,
+		.y = workarea_top,
+		.width = workarea_right - workarea_left,
+		.height = workarea_bottom - workarea_top,
+	};
+	wlr_xwayland_set_workareas(server->xwayland, &workarea, 1);
+}
